unit NDUtil;

interface

uses
  Classes, SysUtils, Graphics;

type
  TNDTokenType = (ttWS,ttComment,ttID,ttString,ttNumber,ttOperator,ttEOLN,ttEOF);
  TNDCommentType = (ctEOLN,ctBraces,ctParenStar);

  TNDParser = class
  protected
    Buf: string; { Buffer for source code }
    BufPtr: PChar; { Pointer to current position in Buf }
    FTokenType: TNDTokenType;
    FTokenName: string;
    FInComment: boolean; // Is parser currently in a comment?
    FCommentType: TNDCommentType; // Type of comment: EOLN, Braces or ParenStar

  public
    procedure LoadFromStream(Stream: TStream);
    procedure LoadFromFile(FileName: string);
    procedure GetToken;

    property TokenType: TNDTokenType read FTokenType write FTokenType;
    property TokenName: string read FTokenName write FTokenName;
    property CommentType: TNDCommentType read FCommentType write FCommentType;
    property InComment: boolean read FInComment write FInComment;
  end; { TNDParser }

  TSyntaxType = (stReserved,stID,stNumber,stString,stOperator,stComment,stWS);
  TSyntaxFont = record
    Bold: boolean;
    Italic: boolean;
    Underline: boolean;
    Color: TColor;
    Name: string;
  end; { TSyntaxFont }

const
  ReservedWords = '|and|array|as|asm|begin|case|class|const|constructor' +
   '|destructor|dispinterface|div|do|downto|else|end|except|exports|file' +
   '|finalization|finally|for|function|goto|if|implementation|in|inherited' +
   '|initialization|inline|interface|is|label|library|mod|nil|not|object|of' +
   '|or|out|packed|procedure|program|property|raise|record|repeat' +
   '|resourcestring|set|shl|shr|string|then|threadvar|to|try|type|unit|until' +
   '|uses|var|while|with|xor|';

var
  SyntaxFonts: array[TSyntaxType] of TSyntaxFont;

  procedure StripToDo(FileName: string;
                      ShowFile: boolean);

implementation

  procedure StripToDo(FileName: string;
                      ShowFile: boolean);

  var
    OutFile: textfile;
    OutFileName: string;
    NDParser: TNDParser;
    Comment: string;
    EndComment: string;
    PrevToken: TNDTokenType;
    S1: string;

  begin { StripToDo }
    If not FileExists(FileName) then begin
      Writeln('Source File specified as parameter does not exist.');
      Exit;
    end; { if }

    OutFileName := FileName;
    If Pos('.',OutFileName) > 0 then begin
      OutFileName := Copy(OutFileName,1,Pos('.',OutFileName) - 1);
    end; { if }
    OutFileName := OutFileName + '.OUT';
    AssignFile(OutFile,OutFileName);
    Rewrite(OutFile);
    NDParser := TNDParser.Create;
    PrevToken := ttEOF;
    With NDParser do try
      LoadFromFile(FileName);
      Repeat
        GetToken;
        If InComment then begin
          Comment := TokenName;
          Repeat
            GetToken;
            Comment := Comment + TokenName;
            If CommentType in [ctBraces,ctParenStar] then begin
              If CommentType = ctBraces then begin
                EndComment := '}';
              end else begin
                EndComment := '*)';
              end; { else }
            end; { if }
          until ((TokenType = ttOperator) and (TokenName = EndComment)) or
           ((TokenType = ttEOLN) and (CommentType = ctEOLN)) or (TokenType = ttEOF);
          If CommentType = ctBraces then begin
            S1 := Copy(Comment,2,Length(Comment));
          end else begin
            S1 := Copy(Comment,3,Length(Comment));
          end; { else }
          While (Length(S1) > 0) and (S1[1] = ' ') do begin
            Delete(S1,1,1);
          end; { while }
          S1 := UpperCase(Copy(S1,1,4));
          If (S1 <> 'TODO') and (S1 <> 'DONE') then begin
            Write(OutFile,Comment);
            If ShowFile then begin
              Write(Comment);
            end; { if }
          end else if PrevToken = ttEOLN then begin
            GetToken;
            If not (TokenType = ttEOLN) then begin
              Write(OutFile,TokenName);
              If ShowFile then begin
                Write(TokenName);
              end; { if }
            end; { if }
          end else begin
            Writeln(OutFile);
            If ShowFile then begin
              Writeln;
            end; { if }
          end; { else }
        end else if TokenType = ttEOLN then begin
          Writeln(OutFile);
          If ShowFile then begin
            Writeln;
          end; { if }
        end else begin
          Write(OutFile,TokenName);
          If ShowFile then begin
            Write(TokenName);
          end; { if }
        end; { else }
        PrevToken := TokenType;
      until (TokenType = ttEOF);
      If ShowFile then begin
        Writeln;
      end; { if }
    finally
      NDParser.Free;
      CloseFile(OutFile);
    end; { with }
    Writeln;
    Writeln(FileName + ' has been converted to ' + OutFileName);
  end;  { StripToDo }


(*****************************************************************************}
( class TNDParser
(*****************************************************************************)

  procedure TNDParser.LoadFromStream(Stream: TStream);

  var
    BufSize: longint;

  begin { LoadFromStream }
    BufSize := Stream.Size - Stream.Position;
    SetLength(Buf,BufSize + 1);
    BufPtr := @Buf[1];
    Stream.ReadBuffer(BufPtr^,BufSize);
    (BufPtr + BufSize)^ := #0; { Put null character on end of buffer }
  end;  { LoadFromStream }

  procedure TNDParser.LoadFromFile(FileName: string);

  var
    Stream: TStream;

  begin { LoadFromFile }
    Stream := TFileStream.Create(FileName,fmOpenRead);
    try
      LoadFromStream(Stream);
    finally
      Stream.Free;
    end; { tryf }
  end;  { LoadFromFile }

  procedure TNDParser.GetToken;

  var
    TokenStart: PChar;

  begin { GetToken }
    TokenStart := BufPtr;
    If InComment then begin // Read until EOC is found
      TokenType := ttComment;
      Case CommentType of
        ctEOLN: begin
          While not (BufPtr^ in [#0,#10,#13]) do begin
            Inc(BufPtr);
          end; { while }
          InComment := false;
        end;
        ctBraces: begin
          If BufPtr^ = #13 then begin
            Inc(BufPtr);
            If BufPtr^ = #10 then begin
              Inc(BufPtr);
            end; { if }
            TokenType := ttEOLN;
          end else if BufPtr^ = #10 then begin
            Inc(BufPtr);
            TokenType := ttEOLN;
          end else begin
            While not (BufPtr^ in [#0,#10,#13,'}']) do begin
              Inc(BufPtr);
            end; { while }
            InComment := (BufPtr^ <> '}');
          end; { else }
        end;
        ctParenStar: begin
          If BufPtr^ = #13 then begin
            Inc(BufPtr);
            If BufPtr^ = #10 then begin
              Inc(BufPtr);
            end; { if }
            TokenType := ttEOLN;
          end else if BufPtr^ = #10 then begin
            Inc(BufPtr);
            TokenType := ttEOLN;
          end else begin
            While not (BufPtr^ in [#0,#10,#13]) do begin
              If BufPtr^ = '*' then begin
                Inc(BufPtr);
                If BufPtr^ = ')' then begin
                  Dec(BufPtr);
                  Break;
                end; { if }
              end else begin
                Inc(BufPtr);
              end; { else }
            end; { while }
            InComment := (BufPtr^ <> '*');
          end; { else }
        end;
      end; { case }
    end else begin
      TokenType := ttOperator;
      Case BufPtr^ of
        #0: begin
          TokenType := ttEOF;
          TokenName := '';
        end;
        #10: begin { End of line }
          Inc(BufPtr);
          TokenType := ttEOLN;
        end;
        #13: begin { End of line }
          Inc(BufPtr);
          If BufPtr^ = #10 then begin
            Inc(BufPtr);
          end; { if }
          TokenType := ttEOLN;
        end;
        #9,#32: begin
          TokenType := ttWS;
          Repeat
            Inc(BufPtr);
          until not (BufPtr^ in [#9,#32]);
        end;
        'A'..'Z','a'..'z','_': begin { ID }
          Repeat
            Inc(BufPtr);
          until not (BufPtr^ in ['0'..'9','A'..'Z','a'..'z','_']);
          TokenType := ttID;
        end;
        '#','''': begin { String }
          TokenType := ttString;
          While true do begin
            Case BufPtr^ of
              '#': begin
                Inc(BufPtr);
                While BufPtr^ in ['0'..'9'] do begin
                  Inc(BufPtr);
                end; { while }
              end;
              '''': begin
                Inc(BufPtr);
                While true do begin
                  Case BufPtr^ of
                    #0,#10,#13: Break; // Error, EOL found inside string
                    '''': begin
                      Inc(BufPtr);
                      If BufPtr^ <> '''' then Break;
                    end;
                  end; { case }
                  Inc(BufPtr);
                end;
              end;
              else Break;
            end; { case }
          end; { while }
        end;
        '$': begin { Hex }
          Inc(BufPtr);
          While BufPtr^ in ['0'..'9','A'..'F','a'..'f'] do begin
            Inc(BufPtr);
          end; { while }
          TokenType := ttNumber;
        end;
        '(': begin { (, (*, (. }
          Inc(BufPtr);
          If BufPtr^ in ['*','.'] then begin
            If BufPtr^ = '*' then begin
              InComment := true;
              CommentType := ctParenStar;
            end; { if }
            Inc(BufPtr);
          end; { if }
        end;
        '*': begin { *, *) }
          Inc(BufPtr);
          If BufPtr^ = ')' then begin { *) }
            Inc(BufPtr);
          end; { if }
        end;
        '.': begin
          Inc(BufPtr);
          If BufPtr^ = '.' then begin { .. }
            Inc(BufPtr);
          end; { if }
        end;
        '/': begin
          Inc(BufPtr);
          If BufPtr^ = '/' then begin { // }
            InComment := true;
            CommentType := ctEOLN;
            Inc(BufPtr);
          end; { if }
        end;
        ':': begin
          Inc(BufPtr);
          If BufPtr^ = '=' then begin { := }
            Inc(BufPtr);
          end; { if }
        end;
        '<': begin
          Inc(BufPtr);
          If BufPtr^ in ['=','>'] then begin { <=, <> }
            Inc(BufPtr);
          end; { if }
        end;
        '>': begin
          Inc(BufPtr);
          If BufPtr^ = '=' then begin { >= }
            Inc(BufPtr);
          end; { if }
        end;
        '0'..'9': begin { Number }
          Repeat
            Inc(BufPtr);
          until not (BufPtr^ in ['0'..'9']);
          TokenType := ttNumber;
          If BufPtr^ = '.' then begin
            Inc(BufPtr);
            If BufPtr^ = '.' then begin { .. - return int token first }
              Dec(BufPtr);
            end else begin { Floating point }
              While BufPtr^ in ['0'..'9'] do begin
                Inc(BufPtr);
              end; { while }
              If BufPtr^ in ['E','e'] then begin
                Inc(BufPtr);
                If BufPtr^ in ['+','-'] then begin
                  Inc(BufPtr);
                end; { if }
                While BufPtr^ in ['0'..'9'] do begin
                  Inc(BufPtr);
                end; { while }
              end; { if }
            end; { else }
          end; { if }
        end;
        '{': begin { Process comment }
          Inc(BufPtr);
          InComment := true;
          CommentType := ctBraces;
        end;
        else begin { Return single operator character } // & ) + , - ; = @ [ ] ^
          Inc(BufPtr);
        end;
      end; { case }
    end; { else }

  { Assign token to TokenName }
    If TokenType = ttEOF then begin
      FTokenName := '';
    end else begin
      SetString(FTokenName,TokenStart,BufPtr - TokenStart);
    end; { if }
  end;  { GetToken }

end.